@@ -5,6 +5,8 @@
 
 G_BEGIN_DECLS
 
+extern guint gegl_debug_flags;
+
 typedef enum {
   GEGL_DEBUG_PROCESS         = 1 << 0,
   GEGL_DEBUG_BUFFER_LOAD     = 1 << 1,
@@ -24,7 +26,7 @@ typedef enum {
  * flags
  */
 #ifdef __GEGL_INIT_C
-static const GDebugKey gegl_debug_keys[] = {
+const GDebugKey gegl_debug_keys[] = {
   { "process",       GEGL_DEBUG_PROCESS},
   { "cache",         GEGL_DEBUG_CACHE},
   { "buffer-load",   GEGL_DEBUG_BUFFER_LOAD},
@@ -45,6 +47,8 @@ static const GDebugKey gegl_debug_keys[] = {
                      GEGL_DEBUG_BUFFER_ALLOC|
                      GEGL_DEBUG_LICENSE},
 };
+#else
+extern GDebugKey gegl_debug_keys[];
 #endif /* __GEGL_INIT_C */
 
 #if defined(__cplusplus) && defined(GEGL_ISO_CXX_VARIADIC_MACROS)
@@ -53,6 +57,12 @@ static const GDebugKey gegl_debug_keys[] = {
 
 #ifdef GEGL_ENABLE_DEBUG
 
+static inline
+gint64 gegl_get_timestamp(void)
+{
+	return g_get_monotonic_time();
+}
+
 #if defined(GEGL_ISO_VARIADIC_MACROS)
 
 /* Use the C99 version; unfortunately, this does not allow us to pass
@@ -72,7 +82,7 @@ static const GDebugKey gegl_debug_keys[] = {
         if (gegl_debug_flags & type)                            \
 	{                                                       \
 	  gchar * _fmt = g_strdup_printf (__VA_ARGS__);         \
-          g_message ("[" #type "]" " %li:"  G_STRLOC ": %s",    \
+          g_message ("[" #type "]" " %"G_GINT64_FORMAT":"  G_STRLOC ": %s",    \
                        gegl_get_timestamp(), _fmt);             \
           g_free (_fmt);                                        \
 	}                                                       \
@@ -88,7 +98,7 @@ static const GDebugKey gegl_debug_keys[] = {
 
 #define GEGL_TIMESTAMP(type,format,a...)        G_STMT_START {  \
         if (gegl_debug_flags & type)                            \
-          { g_message ("[" #type "]" " %li:"  G_STRLOC ": "     \
+          { g_message ("[" #type "]" " %"G_GINT64_FORMAT":"  G_STRLOC ": "     \
                        format, gegl_get_timestamp(), ##a); }    \
                                                    } G_STMT_END
 
@@ -125,7 +135,7 @@ gegl_lookup_debug_string  (guint type)
 static inline void
 GEGL_NOTE (guint type, const char *format, ...)
 {
-  va_alist args;
+  va_list args;
 
   if (gegl_debug_flags & type)
   {
@@ -142,14 +152,14 @@ GEGL_NOTE (guint type, const char *format, ...)
 static inline void
 GEGL_TIMESTAMP (guint type, const char *format, ...)
 {
-  va_alist args;
+  va_list args;
   
   if (gegl_debug_flags & type)
   {
     gchar *formatted;
     va_start (args, format);
     formatted = g_strdup_printf (format, args);
-    g_message ("[ %s ] %li: " G_STRLOC ": %s",
+    g_message ("[ %s ] %"G_GINT64_FORMAT": " G_STRLOC ": %s",
                gegl_lookup_debug_string (type), gegl_get_timestamp(),
                formatted);
     g_free (formatted);
@@ -195,8 +205,6 @@ GEGL_TIMESTAMP (guint type, const char *format, ...)
 
 #endif /* GEGL_ENABLE_DEBUG */
 
-extern guint gegl_debug_flags;
-
 G_END_DECLS
 
 #endif /* __GEGL_DEBUG_H__  */
