@@ -43,6 +43,7 @@
 
 #include "operation/gegl-operation.h"
 #include "operation/gegl-operation-private.h"
+#include "operation/gegl-operation-context-private.h"
 #include "operation/gegl-operations.h"
 #include "operation/gegl-operation-meta.h"
 
@@ -208,9 +209,7 @@ gegl_node_class_init (GeglNodeClass *klass)
     g_signal_new ("invalidated",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__BOXED,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   GEGL_TYPE_RECTANGLE);
 
@@ -218,9 +217,7 @@ gegl_node_class_init (GeglNodeClass *klass)
     g_signal_new ("computed",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__BOXED,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   GEGL_TYPE_RECTANGLE);
 
@@ -228,9 +225,7 @@ gegl_node_class_init (GeglNodeClass *klass)
     g_signal_new ("progress",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__DOUBLE,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 1, G_TYPE_DOUBLE);
 }
 
@@ -1425,6 +1420,8 @@ gegl_node_set_operation_object (GeglNode      *self,
   while (self->pads)
     gegl_node_remove_pad (self, self->pads->data);
 
+  gegl_node_remove_children (self);
+
   gegl_operation_attach (operation, self);
 
   /* FIXME: This should handle all input pads instead of just these 3 */
@@ -1785,12 +1782,20 @@ gegl_node_get_producer (GeglNode     *node,
   GeglNode *ret;
   gpointer pad;
 
+  if (!GEGL_IS_NODE (node))
+    return NULL;
+
   /* XXX: there should be public API to test if a node is
    * really a graph. So that the user of the API knows
    * the internals can be reached through the proxy nops
    */
   if (node->is_graph)
-    node = gegl_node_get_input_proxy (node, "input");
+  {
+    node = gegl_node_get_input_proxy (node, pad_name);
+    if (!GEGL_IS_NODE (node))
+      return NULL;
+    pad_name = "input";
+  }
 
   pad = gegl_node_get_pad (node, pad_name);
 
@@ -1919,7 +1924,10 @@ gegl_node_get_consumers (GeglNode      *node,
   g_return_val_if_fail (output_pad != NULL, 0);
 
   if(node->is_graph)
-    node = gegl_node_get_output_proxy(node, "output");
+    {
+      node = gegl_node_get_output_proxy(node, output_pad);
+      output_pad = "output";
+    }
 
   g_return_val_if_fail (GEGL_IS_NODE (node), 0);
 
@@ -2047,7 +2055,7 @@ gegl_node_get_cache (GeglNode *node)
 
   if (!format)
     {
-      g_warning ("Output of %s has no format", gegl_node_get_debug_name (node));
+      //g_message ("Output of %s has no format", gegl_node_get_debug_name (node));
 
       format = babl_format ("RGBA float");
     }
@@ -2066,9 +2074,11 @@ gegl_node_get_cache (GeglNode *node)
     {
       GeglCache *cache;
 
-      cache = g_object_new (GEGL_TYPE_CACHE,
-                            "format", format,
-                            NULL);
+      cache = g_object_new (
+        GEGL_TYPE_CACHE,
+        "format",      format,
+        "initialized", gegl_operation_context_get_init_output (),
+        NULL);
 
       gegl_object_set_has_forked (G_OBJECT (cache));
       gegl_buffer_set_extent (GEGL_BUFFER (cache), &node->have_rect);
