@@ -11,38 +11,21 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås
  */
 
 #include "config.h"
 
+#include <glib-object.h>
 #include <string.h>
 
-#include <glib-object.h>
-
 #include "gegl.h"
-#include "gegl-types-internal.h"
 #include "gegl-operation-source.h"
-#include "graph/gegl-node.h"
-#include "graph/gegl-pad.h"
+#include "gegl-operation-context.h"
+#include "gegl-config.h"
 
-enum
-{
-  PROP_0,
-  PROP_OUTPUT,
-  PROP_LAST
-};
-
-static void     get_property (GObject      *gobject,
-                              guint         prop_id,
-                              GValue       *value,
-                              GParamSpec   *pspec);
-static void     set_property (GObject      *gobject,
-                              guint         prop_id,
-                              const GValue *value,
-                              GParamSpec   *pspec);
 static gboolean gegl_operation_source_process
                              (GeglOperation        *operation,
                               GeglOperationContext *context,
@@ -59,33 +42,18 @@ static GeglRectangle get_bounding_box          (GeglOperation        *self);
 static GeglRectangle get_required_for_output   (GeglOperation        *operation,
                                                  const gchar         *input_pad,
                                                  const GeglRectangle *roi);
-static GeglRectangle  get_cached_region        (GeglOperation       *operation,
-                                                 const GeglRectangle *roi);
 
 
 static void
 gegl_operation_source_class_init (GeglOperationSourceClass * klass)
 {
-  GObjectClass       *gobject_class   = G_OBJECT_CLASS (klass);
   GeglOperationClass *operation_class = GEGL_OPERATION_CLASS (klass);
 
-  gobject_class->set_property = set_property;
-  gobject_class->get_property = get_property;
-
   operation_class->process = gegl_operation_source_process;
   operation_class->attach  = attach;
-  operation_class->get_cached_region = get_cached_region;
 
   operation_class->get_bounding_box  = get_bounding_box;
   operation_class->get_required_for_output = get_required_for_output;
-
-  g_object_class_install_property (gobject_class, PROP_OUTPUT,
-                                   g_param_spec_object ("output",
-                                                        "Output",
-                                                        "Output pad for generated image buffer.",
-                                                        GEGL_TYPE_BUFFER,
-                                                        G_PARAM_READABLE |
-                                                        GEGL_PARAM_PAD_OUTPUT));
 }
 
 static void
@@ -96,28 +64,35 @@ gegl_operation_source_init (GeglOperationSource *self)
 static void
 attach (GeglOperation *self)
 {
-  GeglOperation *operation    = GEGL_OPERATION (self);
-  GObjectClass  *object_class = G_OBJECT_GET_CLASS (self);
+  GeglOperation *operation = GEGL_OPERATION (self);
+  GParamSpec    *pspec;
 
-  gegl_operation_create_pad (operation,
-                             g_object_class_find_property (object_class,
-                                                           "output"));
+  pspec = g_param_spec_object ("output",
+                               "Output",
+                               "Output pad for generated image buffer.",
+                               GEGL_TYPE_BUFFER,
+                               G_PARAM_READABLE |
+                               GEGL_PARAM_PAD_OUTPUT);
+  gegl_operation_create_pad (operation, pspec);
+  g_param_spec_sink (pspec);
 }
 
-static void
-get_property (GObject    *object,
-              guint       prop_id,
-              GValue     *value,
-              GParamSpec *pspec)
+typedef struct ThreadData
 {
-}
+  GeglOperationSourceClass *klass;
+  GeglOperation            *operation;
+  GeglBuffer               *output;
+  gint                      level;
+  gboolean                  success;
+} ThreadData;
 
 static void
-set_property (GObject      *object,
-              guint         prop_id,
-              const GValue *value,
-              GParamSpec   *pspec)
+thread_process (const GeglRectangle *area,
+                ThreadData          *data)
 {
+  if (!data->klass->process (data->operation,
+                       data->output, area, data->level))
+    data->success = FALSE;
 }
 
 static gboolean
@@ -129,7 +104,7 @@ gegl_operation_source_process (GeglOperation        *operation,
 {
   GeglOperationSourceClass *klass = GEGL_OPERATION_SOURCE_GET_CLASS (operation);
   GeglBuffer               *output;
-  gboolean                  success;
+  gboolean                  success = FALSE;
 
   if (strcmp (output_prop, "output"))
     {
@@ -139,10 +114,30 @@ gegl_operation_source_process (GeglOperation        *operation,
 
   g_assert (klass->process);
   output = gegl_operation_context_get_target (context, "output");
-  success = klass->process (operation, output, result, level);
 
-  if (output == GEGL_BUFFER (operation->node->cache))
-    gegl_cache_computed (operation->node->cache, result);
+  if (gegl_operation_use_threading (operation, result))
+  {
+    ThreadData data;
+
+    data.klass = klass;
+    data.operation = operation;
+    data.output = output;
+    data.level = level;
+    data.success = TRUE;
+
+    gegl_parallel_distribute_area (
+      result,
+      gegl_operation_get_pixels_per_thread (operation),
+      GEGL_SPLIT_STRATEGY_AUTO,
+      (GeglParallelDistributeAreaFunc) thread_process,
+      &data);
+
+    success = data.success;
+  }
+  else
+  {
+    success = klass->process (operation, output, result, level);
+  }
 
   return success;
 }
@@ -166,10 +161,3 @@ get_required_for_output (GeglOperation        *operation,
 {
   return *roi;
 }
-
-static GeglRectangle
-get_cached_region (GeglOperation       *operation,
-                   const GeglRectangle *roi)
-{
-  return *roi;
-}
