@@ -139,7 +139,6 @@ property_enum (pattern, _("Black pattern"),
 property_double (period, _("Black period"), 12.0)
                  value_range (0.0, 200.0)
                  description (_("The number of pixels across one repetition of a base pattern at base resolution."))
-                 description (_("Angle offset for patterns"))
                  ui_meta ("visible", "color-model {white-on-black, black-on-white, cmyk}")
                  ui_meta ("label", "[color-model {white-on-black,"
                                    "              black-on-white} : bw-label,"
@@ -376,7 +375,7 @@ process (GeglOperation       *operation,
 
            for (int c = 0; c < 3; c++)
              out_pixel[c] = gray;
-           out_pixel[3] = 1.0;
+           out_pixel[3] = in_pixel[3];
 
            out_pixel += 4;
            in_pixel  += 4;
@@ -404,7 +403,7 @@ process (GeglOperation       *operation,
 
            for (int c = 0; c < 3; c++)
              out_pixel[c] = gray;
-           out_pixel[3] = 1.0;
+           out_pixel[3] = in_pixel[3];
 
            out_pixel += 4;
            in_pixel  += 4;
@@ -448,7 +447,7 @@ process (GeglOperation       *operation,
                                    o->angleboost,
                                    degrees_to_radians (o->angle4),
                                    o->aa_samples);
-           out_pixel[3] = 1.0;
+           out_pixel[3] = in_pixel[3];
 
            out_pixel += 4;
            in_pixel  += 4;
@@ -616,6 +615,28 @@ gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointFilterClass *point_filter_class;
+  gchar                         *composition =
+    "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "  <node operation='gegl:crop' width='200' height='200'/>"
+    "  <node operation='gegl:over'>"
+    "    <node operation='gegl:newsprint'>"
+    "      <params>"
+    "        <param name='period'>6.0</param>"
+    "        <param name='pattern'>pssquare</param>"
+    "        <param name='color-model'>cmyk</param>"
+    "      </params>"
+    "    </node>"
+    "    <node operation='gegl:load' path='standard-input.png'/>"
+    "  </node>"
+    "  <node operation='gegl:checkerboard'>"
+    "    <params>"
+    "      <param name='color1'>rgb(0.25,0.25,0.25)</param>"
+    "      <param name='color2'>rgb(0.75,0.75,0.75)</param>"
+    "    </params>"
+    "  </node>"    
+    "</gegl>";
+
 
   operation_class = GEGL_OPERATION_CLASS (klass);
   point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);
@@ -630,12 +651,11 @@ gegl_op_class_init (GeglOpClass *klass)
   gegl_operation_class_set_keys (operation_class,
     "name",               "gegl:newsprint",
     "title",              _("Newsprint"),
-    "position-dependent", "true",
     "categories" ,        "render",
-    "reference-hash",     "f680e099d412e28dfa26f9b19e34109f",
-    "description",        _("Digital halftoning with optional modulations. "),
-    "reference-chain",    "load path=images/standard-input.png newsprint period=6.0 pattern=pssquare color-model=cmyk",
     "position-dependent", "true",
+    "reference-hash",     "2792e6d5aec5c2f80e45aea3a610d1f6",
+    "reference-composition", composition,
+    "description",        _("Digital halftoning with optional modulations. "),
     NULL);
 }
 
