@@ -17,9 +17,6 @@
 #include "lzw.h"
 #include "nsgif.h"
 
-/** Maximum colour table size */
-#define NSGIF_MAX_COLOURS 256
-
 /** Default minimum allowable frame delay in cs. */
 #define NSGIF_FRAME_DELAY_MIN 2
 
@@ -35,7 +32,7 @@ typedef struct nsgif_frame {
 	struct nsgif_frame_info info;
 
 	/** offset (in bytes) to the GIF frame data */
-	uint32_t frame_pointer;
+	uint32_t frame_offset;
 	/** whether the frame has previously been decoded. */
 	bool decoded;
 	/** whether the frame is totally opaque */
@@ -43,9 +40,15 @@ typedef struct nsgif_frame {
 	/** whether a full image redraw is required */
 	bool redraw_required;
 
+	/** Amount of LZW data found in scan */
+	uint32_t lzw_data_length;
+
 	/** the index designating a transparent pixel */
 	uint32_t transparency_index;
 
+	/** offset to frame colour table */
+	uint32_t colour_table_offset;
+
 	/* Frame flags */
 	uint32_t flags;
 } nsgif_frame;
@@ -72,8 +75,11 @@ struct nsgif {
 	uint32_t frame;
 	/** current frame decoded to bitmap */
 	uint32_t decoded_frame;
+
 	/** currently decoded image; stored as bitmap from bitmap_create callback */
 	nsgif_bitmap_t *frame_image;
+	/** Row span of frame_image in pixels. */
+	uint32_t rowspan;
 
 	/** Minimum allowable frame delay. */
 	uint16_t delay_min;
@@ -87,6 +93,12 @@ struct nsgif {
 	/** number of frames partially decoded */
 	uint32_t frame_count_partial;
 
+	/**
+	 * Whether all the GIF data has been supplied, or if there may be
+	 * more to come.
+	 */
+	bool data_complete;
+
 	/** pointer to GIF data */
 	const uint8_t *buf;
 	/** current index into GIF data */
@@ -100,11 +112,9 @@ struct nsgif {
 	uint32_t bg_index;
 	/** image aspect ratio (ignored) */
 	uint32_t aspect_ratio;
-	/** size of colour table (in entries) */
+	/** size of global colour table (in entries) */
 	uint32_t colour_table_size;
 
-	/** whether the GIF has a global colour table */
-	bool global_colours;
 	/** current colour table */
 	uint32_t *colour_table;
 	/** Client's colour component order. */
@@ -223,6 +233,11 @@ static inline uint32_t* nsgif__bitmap_get(
 		return NULL;
 	}
 
+	gif->rowspan = gif->info.width;
+	if (gif->bitmap.get_rowspan) {
+		gif->rowspan = gif->bitmap.get_rowspan(gif->frame_image);
+	}
+
 	/* Get the frame data */
 	assert(gif->bitmap.get_buffer);
 	return (void *)gif->bitmap.get_buffer(gif->frame_image);
@@ -280,6 +295,9 @@ static void nsgif__record_frame(
 		struct nsgif *gif,
 		const uint32_t *bitmap)
 {
+	size_t pixel_bytes = sizeof(*bitmap);
+	size_t height = gif->info.height;
+	size_t width  = gif->info.width;
 	uint32_t *prev_frame;
 
 	if (gif->decoded_frame == NSGIF_FRAME_INVALID ||
@@ -295,7 +313,7 @@ static void nsgif__record_frame(
 
 	if (gif->prev_frame == NULL) {
 		prev_frame = realloc(gif->prev_frame,
-				gif->info.width * gif->info.height * 4);
+				width * height * pixel_bytes);
 		if (prev_frame == NULL) {
 			return;
 		}
@@ -303,7 +321,7 @@ static void nsgif__record_frame(
 		prev_frame = gif->prev_frame;
 	}
 
-	memcpy(prev_frame, bitmap, gif->info.width * gif->info.height * 4);
+	memcpy(prev_frame, bitmap, width * height * pixel_bytes);
 
 	gif->prev_frame  = prev_frame;
 	gif->prev_index  = gif->decoded_frame;
@@ -314,10 +332,11 @@ static nsgif_error nsgif__recover_frame(
 		uint32_t *bitmap)
 {
 	const uint32_t *prev_frame = gif->prev_frame;
-	unsigned height = gif->info.height;
-	unsigned width  = gif->info.width;
+	size_t pixel_bytes = sizeof(*bitmap);
+	size_t height = gif->info.height;
+	size_t width  = gif->info.width;
 
-	memcpy(bitmap, prev_frame, height * width * sizeof(*bitmap));
+	memcpy(bitmap, prev_frame, height * width * pixel_bytes);
 
 	return NSGIF_OK;
 }
@@ -463,7 +482,7 @@ static nsgif_error nsgif__decode_complex(
 		uint32_t *frame_scanline;
 
 		frame_scanline = frame_data + offset_x +
-				(y + offset_y) * gif->info.width;
+				(y + offset_y) * gif->rowspan;
 
 		x = width;
 		while (x > 0) {
@@ -594,7 +613,9 @@ static inline nsgif_error nsgif__decode(
 	uint32_t transparency_index = frame->transparency_index;
 	uint32_t *restrict colour_table = gif->colour_table;
 
-	if (interlace == false && width == gif->info.width && offset_x == 0) {
+	if (interlace == false && offset_x == 0 &&
+			width == gif->info.width &&
+			width == gif->rowspan) {
 		ret = nsgif__decode_simple(gif, height, offset_y,
 				data, transparency_index,
 				frame_data, colour_table);
@@ -605,6 +626,11 @@ static inline nsgif_error nsgif__decode(
 				frame_data, colour_table);
 	}
 
+	if (gif->data_complete && ret == NSGIF_ERR_END_OF_DATA) {
+		/* This is all the data there is, so make do. */
+		ret = NSGIF_OK;
+	}
+
 	return ret;
 }
 
@@ -620,9 +646,14 @@ static void nsgif__restore_bg(
 		struct nsgif_frame *frame,
 		uint32_t *bitmap)
 {
+	size_t pixel_bytes = sizeof(*bitmap);
+
 	if (frame == NULL) {
+		size_t width  = gif->info.width;
+		size_t height = gif->info.height;
+
 		memset(bitmap, NSGIF_TRANSPARENT_COLOUR,
-				gif->info.width * gif->info.height * sizeof(*bitmap));
+				width * height * pixel_bytes);
 	} else {
 		uint32_t width  = frame->info.rect.x1 - frame->info.rect.x0;
 		uint32_t height = frame->info.rect.y1 - frame->info.rect.y0;
@@ -643,7 +674,7 @@ static void nsgif__restore_bg(
 				uint32_t *scanline = bitmap + offset_x +
 						(offset_y + y) * gif->info.width;
 				memset(scanline, NSGIF_TRANSPARENT_COLOUR,
-						width * sizeof(*bitmap));
+						width * pixel_bytes);
 			}
 		} else {
 			for (uint32_t y = 0; y < height; y++) {
@@ -1031,6 +1062,38 @@ static nsgif_error nsgif__parse_image_descriptor(
 	return NSGIF_OK;
 }
 
+/**
+ * Extract a GIF colour table into a LibNSGIF colour table buffer.
+ *
+ * \param[in] colour_table          The colour table to populate.
+ * \param[in] layout                la.
+ * \param[in] colour_table_entries  The number of colour table entries.
+ * \param[in] Data                  Raw colour table data.
+ */
+static void nsgif__colour_table_decode(
+		uint32_t colour_table[NSGIF_MAX_COLOURS],
+		const struct nsgif_colour_layout *layout,
+		size_t colour_table_entries,
+		const uint8_t *data)
+{
+	uint8_t *entry = (uint8_t *)colour_table;
+
+	while (colour_table_entries--) {
+		/* Gif colour map contents are r,g,b.
+		 *
+		 * We want to pack them bytewise into the colour table,
+		 * according to the client colour layout.
+		 */
+
+		entry[layout->r] = *data++;
+		entry[layout->g] = *data++;
+		entry[layout->b] = *data++;
+		entry[layout->a] = 0xff;
+
+		entry += sizeof(uint32_t);
+	}
+}
+
 /**
  * Extract a GIF colour table into a LibNSGIF colour table buffer.
  *
@@ -1041,42 +1104,25 @@ static nsgif_error nsgif__parse_image_descriptor(
  * \param[in] decode                Whether to decode the colour table.
  * \return NSGIF_OK on success, appropriate error otherwise.
  */
-static nsgif_error nsgif__colour_table_extract(
-		struct nsgif *gif,
+static inline nsgif_error nsgif__colour_table_extract(
 		uint32_t colour_table[NSGIF_MAX_COLOURS],
 		const struct nsgif_colour_layout *layout,
 		size_t colour_table_entries,
-		const uint8_t **pos,
+		const uint8_t *data,
+		size_t data_len,
+		size_t *used,
 		bool decode)
 {
-	const uint8_t *data = *pos;
-	size_t len = gif->buf + gif->buf_len - data;
-
-	if (len < colour_table_entries * 3) {
+	if (data_len < colour_table_entries * 3) {
 		return NSGIF_ERR_END_OF_DATA;
 	}
 
 	if (decode) {
-		int count = colour_table_entries;
-		uint8_t *entry = (uint8_t *)colour_table;
-
-		while (count--) {
-			/* Gif colour map contents are r,g,b.
-			 *
-			 * We want to pack them bytewise into the colour table,
-			 * according to the client colour layout.
-			 */
-
-			entry[layout->r] = *data++;
-			entry[layout->g] = *data++;
-			entry[layout->b] = *data++;
-			entry[layout->a] = 0xff;
-
-			entry += sizeof(uint32_t);
-		}
+		nsgif__colour_table_decode(colour_table, layout,
+				colour_table_entries, data);
 	}
 
-	*pos += colour_table_entries * 3;
+	*used = colour_table_entries * 3;
 	return NSGIF_OK;
 }
 
@@ -1098,6 +1144,9 @@ static nsgif_error nsgif__parse_colour_table(
 		bool decode)
 {
 	nsgif_error ret;
+	const uint8_t *data = *pos;
+	size_t len = gif->buf + gif->buf_len - data;
+	size_t used_bytes;
 
 	assert(gif != NULL);
 	assert(frame != NULL);
@@ -1107,15 +1156,25 @@ static nsgif_error nsgif__parse_colour_table(
 		return NSGIF_OK;
 	}
 
-	ret = nsgif__colour_table_extract(gif,
+	if (decode == false) {
+		frame->colour_table_offset = *pos - gif->buf;
+	}
+
+	ret = nsgif__colour_table_extract(
 			gif->local_colour_table, &gif->colour_layout,
 			2 << (frame->flags & NSGIF_COLOUR_TABLE_SIZE_MASK),
-			pos, decode);
+			data, len, &used_bytes, decode);
 	if (ret != NSGIF_OK) {
 		return ret;
 	}
+	*pos += used_bytes;
+
+	if (decode) {
+		gif->colour_table = gif->local_colour_table;
+	} else {
+		frame->info.local_palette = true;
+	}
 
-	gif->colour_table = gif->local_colour_table;
 	return NSGIF_OK;
 }
 
@@ -1177,16 +1236,19 @@ static nsgif_error nsgif__parse_image_data(
 		len--;
 
 		while (block_size != 1) {
-			if (len < 1) return NSGIF_ERR_END_OF_DATA;
+			if (len < 1) {
+				return NSGIF_ERR_END_OF_DATA;
+			}
 			block_size = data[0] + 1;
 			/* Check if the frame data runs off the end of the file */
 			if (block_size > len) {
-				block_size = len;
-				return NSGIF_OK;
+				frame->lzw_data_length += len;
+				return NSGIF_ERR_END_OF_DATA;
 			}
 
 			len -= block_size;
 			data += block_size;
+			frame->lzw_data_length += block_size;
 		}
 
 		*pos = data;
@@ -1222,13 +1284,16 @@ static struct nsgif_frame *nsgif__get_frame(
 
 		frame = &gif->frames[frame_idx];
 
-		frame->transparency_index = NSGIF_NO_TRANSPARENCY;
-		frame->frame_pointer = gif->buf_pos;
+		frame->info.local_palette = false;
 		frame->info.transparency = false;
-		frame->redraw_required = false;
 		frame->info.display = false;
 		frame->info.disposal = 0;
 		frame->info.delay = 10;
+
+		frame->transparency_index = NSGIF_NO_TRANSPARENCY;
+		frame->frame_offset = gif->buf_pos;
+		frame->redraw_required = false;
+		frame->lzw_data_length = 0;
 		frame->decoded = false;
 	}
 
@@ -1261,7 +1326,7 @@ static nsgif_error nsgif__process_frame(
 	end = gif->buf + gif->buf_len;
 
 	if (decode) {
-		pos = gif->buf + frame->frame_pointer;
+		pos = gif->buf + frame->frame_offset;
 
 		/* Ensure this frame is supposed to be decoded */
 		if (frame->info.display == false) {
@@ -1536,7 +1601,7 @@ static nsgif_error nsgif__parse_logical_screen_descriptor(
 
 	gif->info.width = data[0] | (data[1] << 8);
 	gif->info.height = data[2] | (data[3] << 8);
-	gif->global_colours = data[4] & NSGIF_COLOUR_TABLE_MASK;
+	gif->info.global_palette = data[4] & NSGIF_COLOUR_TABLE_MASK;
 	gif->colour_table_size = 2 << (data[4] & NSGIF_COLOUR_TABLE_SIZE_MASK);
 	gif->bg_index = data[5];
 	gif->aspect_ratio = data[6];
@@ -1556,6 +1621,10 @@ nsgif_error nsgif_data_scan(
 	nsgif_error ret;
 	uint32_t frames;
 
+	if (gif->data_complete) {
+		return NSGIF_ERR_DATA_COMPLETE;
+	}
+
 	/* Initialize values */
 	gif->buf_len = size;
 	gif->buf = data;
@@ -1630,16 +1699,20 @@ nsgif_error nsgif_data_scan(
 	 */
 	if (gif->global_colour_table[0] == NSGIF_PROCESS_COLOURS) {
 		/* Check for a global colour map signified by bit 7 */
-		if (gif->global_colours) {
-			ret = nsgif__colour_table_extract(gif,
+		if (gif->info.global_palette) {
+			size_t remaining = gif->buf + gif->buf_len - nsgif_data;
+			size_t used;
+
+			ret = nsgif__colour_table_extract(
 					gif->global_colour_table,
 					&gif->colour_layout,
 					gif->colour_table_size,
-					&nsgif_data, true);
+					nsgif_data, remaining, &used, true);
 			if (ret != NSGIF_OK) {
 				return ret;
 			}
 
+			nsgif_data += used;
 			gif->buf_pos = (nsgif_data - gif->buf);
 		} else {
 			/* Create a default colour table with the first two
@@ -1659,9 +1732,11 @@ nsgif_error nsgif_data_scan(
 			entry[gif->colour_layout.g] = 0xFF;
 			entry[gif->colour_layout.b] = 0xFF;
 			entry[gif->colour_layout.a] = 0xFF;
+
+			gif->colour_table_size = 2;
 		}
 
-		if (gif->global_colours &&
+		if (gif->info.global_palette &&
 		    gif->bg_index < gif->colour_table_size) {
 			size_t bg_idx = gif->bg_index;
 			gif->info.background = gif->global_colour_table[bg_idx];
@@ -1691,6 +1766,32 @@ nsgif_error nsgif_data_scan(
 	return ret;
 }
 
+/* exported function documented in nsgif.h */
+void nsgif_data_complete(
+		nsgif_t *gif)
+{
+	if (gif->data_complete == false) {
+		uint32_t start = gif->info.frame_count;
+		uint32_t end = gif->frame_count_partial;
+
+		for (uint32_t f = start; f < end; f++) {
+			nsgif_frame *frame = &gif->frames[f];
+
+			if (frame->lzw_data_length > 0) {
+				frame->info.display = true;
+				gif->info.frame_count = f + 1;
+
+				if (f == 0) {
+					frame->info.transparency = true;
+				}
+				break;
+			}
+		}
+	}
+
+	gif->data_complete = true;
+}
+
 static void nsgif__redraw_rect_extend(
 		const nsgif_rect_t *frame,
 		nsgif_rect_t *redraw)
@@ -1714,7 +1815,7 @@ static void nsgif__redraw_rect_extend(
 }
 
 static uint32_t nsgif__frame_next(
-		nsgif_t *gif,
+		const nsgif_t *gif,
 		bool partial,
 		uint32_t frame)
 {
@@ -1731,7 +1832,7 @@ static uint32_t nsgif__frame_next(
 }
 
 static nsgif_error nsgif__next_displayable_frame(
-		nsgif_t *gif,
+		const nsgif_t *gif,
 		uint32_t *frame,
 		uint32_t *delay)
 {
@@ -1739,7 +1840,11 @@ static nsgif_error nsgif__next_displayable_frame(
 
 	do {
 		next = nsgif__frame_next(gif, false, next);
-		if (next == *frame || next == NSGIF_FRAME_INVALID) {
+		if (next <= *frame && *frame != NSGIF_FRAME_INVALID &&
+				gif->data_complete == false) {
+			return NSGIF_ERR_END_OF_DATA;
+
+		} else if (next == *frame || next == NSGIF_FRAME_INVALID) {
 			return NSGIF_ERR_FRAME_DISPLAY;
 		}
 
@@ -1807,21 +1912,26 @@ nsgif_error nsgif_frame_prepare(
 		gif->loop_count++;
 	}
 
-	if (gif->info.frame_count == 1) {
-		delay = NSGIF_INFINITE;
+	if (gif->data_complete) {
+		/* Check for last frame, which has infinite delay. */
 
-	} else if (gif->info.loop_max != 0) {
-		uint32_t frame_next = frame;
-		ret = nsgif__next_displayable_frame(gif, &frame_next, NULL);
-		if (ret != NSGIF_OK) {
-			return ret;
-		}
+		if (gif->info.frame_count == 1) {
+			delay = NSGIF_INFINITE;
+		} else if (gif->info.loop_max != 0) {
+			uint32_t frame_next = frame;
 
-		if (frame_next < frame) {
-			if (nsgif__animation_complete(
-					gif->loop_count + 1,
-					gif->info.loop_max)) {
-				delay = NSGIF_INFINITE;
+			ret = nsgif__next_displayable_frame(gif,
+					&frame_next, NULL);
+			if (ret != NSGIF_OK) {
+				return ret;
+			}
+
+			if (gif->data_complete && frame_next < frame) {
+				if (nsgif__animation_complete(
+						gif->loop_count + 1,
+						gif->info.loop_max)) {
+					delay = NSGIF_INFINITE;
+				}
 			}
 		}
 	}
@@ -1895,6 +2005,43 @@ const nsgif_frame_info_t *nsgif_get_frame_info(
 	return &gif->frames[frame].info;
 }
 
+/* exported function documented in nsgif.h */
+void nsgif_global_palette(
+		const nsgif_t *gif,
+		uint32_t table[NSGIF_MAX_COLOURS],
+		size_t *entries)
+{
+	size_t len = sizeof(*table) * NSGIF_MAX_COLOURS;
+
+	memcpy(table, gif->global_colour_table, len);
+	*entries = gif->colour_table_size;
+}
+
+/* exported function documented in nsgif.h */
+bool nsgif_local_palette(
+		const nsgif_t *gif,
+		uint32_t frame,
+		uint32_t table[NSGIF_MAX_COLOURS],
+		size_t *entries)
+{
+	const nsgif_frame *f;
+
+	if (frame >= gif->frame_count_partial) {
+		return false;
+	}
+
+	f = &gif->frames[frame];
+	if (f->info.local_palette == false) {
+		return false;
+	}
+
+	*entries = 2 << (f->flags & NSGIF_COLOUR_TABLE_SIZE_MASK);
+	nsgif__colour_table_decode(table, &gif->colour_layout,
+			*entries, gif->buf + f->colour_table_offset);
+
+	return true;
+}
+
 /* exported function documented in nsgif.h */
 const char *nsgif_strerror(nsgif_error err)
 {
@@ -1906,6 +2053,7 @@ const char *nsgif_strerror(nsgif_error err)
 		[NSGIF_ERR_DATA_FRAME]    = "Invalid frame data",
 		[NSGIF_ERR_FRAME_COUNT]   = "Excessive number of frames",
 		[NSGIF_ERR_END_OF_DATA]   = "Unexpected end of GIF source data",
+		[NSGIF_ERR_DATA_COMPLETE] = "Can't add data to completed GIF",
 		[NSGIF_ERR_FRAME_DISPLAY] = "Frame can't be displayed",
 		[NSGIF_ERR_ANIMATION_END] = "Animation complete",
 	};
